/* stdio.h
 *
 *	(C) Copyright Apr 14 1995, Edmond J. Breen.
 *		   ALL RIGHTS RESERVED.
 * This code may be copied for personal, non-profit use only.
 *
 */

#ifndef STDIOH_
#define STDIOH_

#include <stdarg.h>   /* this must get fixed */

#define _need_NULL
#define _need_size_t
#define _need_eic_ptr

#include "sys/stdtypes.h"

#undef _need_size_t
#undef _need_NULL
#undef _need_eic_ptr


#define BUFSIZ 512
#define FILENAME_MAX 64   /* there is no limit here! */
#define FOPEN_MAX    63
#define L_tmpnam     16
#define TMP_MAX      32


#define SEEK_SET        0
#define SEEK_CUR        1
#define SEEK_END        2

#define _IOFBF          1  /* fully buffered */
#define _IOLBF          2  /* line buffered */
#define _IONBF          3  /* none buffered */


#define EOF     (-1)	/* end of file indicator */
#define _EOF     0x40	/* end of file has occured */
#define _ERR     0x80	/* error has occured */


typedef unsigned long fpos_t;

#ifndef _EiC

typedef struct {
    ptr_t _ptr;              /* next char position */
    int _cnt;                /* number of chars left */
    ptr_t _base;             /* location of buffer */
    unsigned short _flag;    /* mode of file access */
    short _fd;               /* file descriptor */
    unsigned long _pid;      /* process id number, if opened as a pipe */
    unsigned  _bsize;        /* buffer size */
    ptr_t _tmpnam;           /* temporary file */
    char _sbuf[4];           /* small buffer, used for unbuffered */
    /* I/O */
    unsigned _entry;         /* entry number: used for purges */ 
}FILE;

extern FILE _IoB[FOPEN_MAX];

#else

typedef struct {
    unsigned char *_ptr;     /* next char position */
    int _cnt;                /* number of chars left */
    unsigned char *_base;    /* location of buffer */
    unsigned short _flag;    /* mode of file access */
    short _fd;               /* file descriptor */
    unsigned long _pid;      /* process id number, if opened as a pipe */
    unsigned  _bsize;        /* buffer size */
    char *_tmpnam;           /* temporary file */
    unsigned char _sbuf[4];	 /* small buffer, used for unbuffered */
    /* I/O */
    unsigned _entry;         /* entry number: used for purges */ 
}FILE;

FILE *_IoB;

#endif


#define stdin   (&_IoB[0])
#define stdout  (&_IoB[1])
#define stderr  (&_IoB[2])

/* Macros *****/
#define fsetpos(x,y)  fseek(x,*(y),SEEK_SET)
#define fgetpos(x,y)  ((*(y) = ftell(x)) >= 0 ? 0: 1)
#define feof(x)       (((x)->_flag & _EOF) != 0)
#define ferror(x)     ((x)->_flag & _ERR)
#define puts(x)       (fputs(x,stdout) < 0 || fputc('\n',stdout) < 0 \
		       ? EOF : 0)
#define rewind(x)  (void)fseek(x,0L,SEEK_SET)
#define vprintf(x,y)  vfprintf(stdout,x,y)

#define setbuf(x,y) do { char *b = (y); \
    setvbuf(x, b, b ? _IOFBF : _IONBF, BUFSIZ); } while (0)

#define getchar()     fgetc(stdin)
#define getc(x)       fgetc(x)
#define putchar(x)    fputc(x,stdout)
#define putc(x,y)     fputc(x,y)

/** Prototypes */
int sprintf(char *buf, const char * fmt, ...);
int printf(const char *fmt, ...);
int fprintf(FILE * fp,const char *fmt, ...);

int vfprintf(FILE * fp,const char *fmt, va_list args);
int vsprintf(char * str,const char *fmt, va_list args);

/** PROTOTYPES from fopen.c **/
long ftell(FILE *fp);
int ungetc(int c, FILE *fp);
char * gets(char *s);
int fgetc(FILE * fp);
int fputc(int c, FILE * fp);
int fputs(const char *s, FILE *fp);
char * fgets(char *s, int n, FILE *fp);
size_t fread(void *buf, size_t elsize, size_t nelems, FILE * fp);
size_t fwrite(const void *buf, size_t elsize, size_t nelems, FILE * fp);
FILE *freopen(const char *name, const char *mode, FILE *fp);
FILE *fopen(const char *name, const char *mode);
int fflush(FILE * fp);
int fclose(FILE * fp);
int fscanf(FILE *fp, const char *fmt, ...);
int scanf(const char *fmt, ...);
int sscanf(const char *str,const char *fmt, ...);
int setvbuf(FILE *fp, char *buf, int mode, size_t size);
char * tmpnam(char*s);
FILE * tmpfile(void);
int remove(const char *fname);
int rename(const char *oldname, const char *newname);
int fseek(FILE *fp, long off, int origin);

void clearerr(FILE *fp);
void perror(const char *msg);


/* UNIX system stuff */
FILE *popen( const char *cmd, const char *mode);
int pclose( FILE *fp);


#ifdef _EiC
     FILE * _get_IoB(void);
     _IoB = _get_IoB();
#else

     extern size_t _NextFopenEntry;
#define fopen_NextEntryNum()   _NextFopenEntry
     void _ffexit(size_t entry);

#define fflush 	_Ufflush
#define fclose  _Ufclose
#define ungetc  _Uungetc
#define fopen   _Ufopen
#define freopen _Ufreopen

     int _Ufflush(FILE * fp);
     int _Ufclose(FILE * fp);
     FILE *_Ufreopen(const char *name, const char *mode, FILE *fp);
     FILE *_Ufopen(const char *name, const char *mode);
     int _Uungetc(int c, FILE *fp);


#endif



#endif

